@@ -3,14 +3,12 @@ use std::path::PathBuf;
 use clap::{Command, Arg, ArgMatches};
 
 use sphinx_lang::frontend;
-use sphinx_lang::BuildErrors;
 use sphinx_lang::source::{ModuleSource, SourceText};
 use sphinx_lang::parser::expr::Expr;
 use sphinx_lang::parser::stmt::{Stmt, StmtMeta};
 use sphinx_lang::codegen::{Program, CompiledProgram};
-use sphinx_lang::runtime::VirtualMachine;
-use sphinx_lang::runtime::gc::GC;
-use sphinx_lang::runtime::module::{Module, GlobalEnv};
+use sphinx_lang::runtime::{Module, VirtualMachine, GC};
+use sphinx_lang::runtime::module::GlobalEnv;
 use sphinx_lang::runtime::strings::StringInterner;
 use sphinx_lang::debug::symbol::resolver::BufferedResolver;
 use sphinx_lang::stdlib;
@@ -80,7 +78,7 @@ fn main() {
         unimplemented!()
     }
     else if args.is_present("interactive") {
-        if let Some(build) = build_program(&args, name, &source) {
+        if let Some(build) = build_program(&source) {
             let program = Program::load(build.program);
             
             let repl_env = GC::allocate(stdlib::prelude_env());
@@ -98,7 +96,7 @@ fn main() {
             Repl::new(version.to_string(), repl_env).run()
         }
     }
-    else if let Some(build) = build_program(&args, name, &source) {
+    else if let Some(build) = build_program(&source) {
         let program = Program::load(build.program);
         
         let main_env = GC::allocate(stdlib::prelude_env());
@@ -116,26 +114,10 @@ fn main() {
 }
 
 
-fn build_program(_args: &ArgMatches, name: &str, source: &ModuleSource) -> Option<CompiledProgram> {
-    // build module
+fn build_program(source: &ModuleSource) -> Option<CompiledProgram> {
     match sphinx_lang::build_module(source) {
-        Err(error) => {
-            match error {
-                BuildErrors::Source(error) => {
-                    println!("Error reading source: {}.", error);
-                }
-                
-                BuildErrors::Syntax(errors) => {
-                    println!("Errors in file \"{}\":\n", name);
-                    frontend::print_source_errors(source, &errors);
-                }
-                
-                BuildErrors::Compile(errors) => {
-                    println!("Errors in file \"{}\":\n", name);
-                    frontend::print_source_errors(source, &errors);
-                }
-            }
-            
+        Err(errors) => {
+            sphinx_lang::print_build_errors(&errors, source);
             None
         },
         
