@@ -1,20 +1,19 @@
 #![allow(unused_variables)]
 
-use log;
 use std::iter;
-use std::mem;
-
 use crate::language::{IntType, FloatType, InternSymbol};
 use crate::parser::stmt::{StmtMeta, Stmt, Label, StmtList, ControlFlow};
 use crate::parser::expr::{Expr, ExprMeta, ExprBlock, ConditionalBranch};
 use crate::parser::primary::{Atom, Primary, AccessItem};
 use crate::parser::lvalue::{Assignment, Declaration, LValue, DeclType};
-use crate::parser::fundefs::{FunctionDef, SignatureDef, ParamDef, DefaultDef};
+use crate::parser::fundefs::{FunctionDef, SignatureDef};
 use crate::runtime::vm::LocalIndex;
 use crate::runtime::types::operator::{UnaryOp, BinaryOp, Arithmetic, Bitwise, Shift, Comparison, Logical};
 use crate::runtime::strings::{StringInterner};
 use crate::debug::symbol::{DebugSymbol, ChunkSymbols, DebugSymbolTable};
 
+mod scope;
+
 pub mod chunk;
 pub mod consts;
 pub mod opcodes;
@@ -25,7 +24,7 @@ pub use chunk::{UnloadedProgram, Program, ProgramData, Chunk, ChunkID};
 pub use consts::{ConstID, Constant};
 pub use errors::{CompileResult, CompileError, ErrorKind};
 
-use opcodes::*;
+use scope::{ScopeTracker, LocalName, UpvalueTarget};
 use chunk::{ChunkBuilder, ChunkInfo, ChunkBuf};
 use consts::{UnloadedSignature, UnloadedParam};
 
@@ -107,166 +106,6 @@ const fn get_jump_opcode(jump: Jump, offset: JumpOffset) -> OpCode {
 }
 
 
-
-// Scope Tracking
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-enum LocalName {
-    // local variable names defined by AST string symbols
-    Symbol(InternSymbol),
-    
-    // special local variables
-    Receiver,  // inside a function call, this refers to the object that was called
-    NArgs,     // inside a function call, the number of arguments passed at the call site
-}
-
-#[derive(Debug)]
-struct Local {
-    decl: DeclType,
-    name: LocalName,
-    index: LocalIndex,
-}
-
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-enum ScopeTag {
-    Block,
-    Loop,
-    Branch,
-    Function,
-    Class,
-}
-
-
-#[derive(Debug)]
-struct Scope {
-    tag: ScopeTag,
-    depth: usize,
-    chunk: Chunk,
-    symbol: Option<DebugSymbol>,
-    
-    frame: Option<LocalIndex>,
-    locals: Vec<Local>,
-}
-
-impl Scope {
-    fn debug_symbol(&self) -> Option<&DebugSymbol> {
-        self.symbol.as_ref()
-    }
-    
-    fn last_index(&self) -> Option<LocalIndex> {
-        self.locals.last().map_or(self.frame, |local| Some(local.index))
-    }
-    
-    fn find_local(&self, name: &LocalName) -> Option<&Local> {
-        self.locals.iter().find(|local| local.name == *name)
-    }
-    
-    fn find_local_mut(&mut self, name: &LocalName) -> Option<&mut Local> {
-        self.locals.iter_mut().find(|local| local.name == *name)
-    }
-    
-    fn push_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<&Local> {
-        let index = self.last_index().map_or(
-            Ok(0),
-            |index| index.checked_add(1)
-                .ok_or_else(|| CompileError::from(ErrorKind::LocalVariableLimit))
-        )?;
-        
-        let local = Local {
-            decl, name, index,
-        };
-        self.locals.push(local);
-        Ok(self.locals.last().unwrap())
-    }
-    
-    fn is_frame_start(&self) -> bool {
-        self.frame.is_none()
-    }
-}
-
-#[derive(Debug)]
-struct ScopeTracker {
-    scopes: Vec<Scope>,
-}
-
-impl ScopeTracker {
-    fn new() -> Self {
-        Self { scopes: Vec::new() }
-    }
-    
-    fn is_global_scope(&self) -> bool {
-        self.scopes.is_empty()
-    }
-    
-    fn local_scope(&self) -> Option<&Scope> {
-        self.scopes.last()
-    }
-    
-    fn local_scope_mut(&mut self) -> Option<&mut Scope> {
-        self.scopes.last_mut()
-    }
-    
-    fn push_scope(&mut self, tag: ScopeTag, chunk: Chunk, symbol: Option<&DebugSymbol>) {
-        let frame;
-        if tag == ScopeTag::Function {
-            frame = None; // inside a function, locals are indexed from the start of the frame
-        } else {
-            frame = self.local_scope().and_then(|scope| scope.last_index());
-        }
-        
-        let scope = Scope {
-            tag, 
-            chunk,
-            symbol: symbol.copied(),
-            depth: self.scopes.len(),
-            frame,
-            locals: Vec::new(),
-        };
-        self.scopes.push(scope);
-    }
-    
-    fn pop_scope(&mut self) -> Scope {
-        self.scopes.pop().expect("pop global scope")
-    }
-    
-    fn insert_local(&mut self, decl: DeclType, name: LocalName) -> CompileResult<()> {
-        let local_scope = self.local_scope_mut().expect("insert local in global scope");
-        
-        // see if this local already exists in the current scope
-        if let Some(mut local) = local_scope.find_local_mut(&name) {
-            (*local).decl = decl; // redeclare with new mutability
-        } else {
-            local_scope.push_local(decl, name)?;
-        }
-        Ok(())
-    }
-    
-    fn iter_scopes(&self) -> impl Iterator<Item=&Scope> {
-        self.scopes.iter().rev()
-    }
-    
-    // find the nearest local in scopes that allow nonlocal assignment
-    fn resolve_local(&self, name: &LocalName) -> Option<&Local> {
-        for scope in self.iter_scopes() {
-            let local = scope.find_local(name);
-            if local.is_some() {
-                return local;
-            }
-            if scope.is_frame_start() {
-                break;
-            }
-        }
-        None
-    }
-    
-    // without the nonlocal keyword, that is
-    fn can_assign_global(&self) -> bool {
-        self.scopes.iter().skip(1).all(|scope| !scope.is_frame_start())
-    }
-}
-
-
-
 /// Output container
 #[derive(Debug)]
 pub struct CompiledProgram {
@@ -274,8 +113,8 @@ pub struct CompiledProgram {
     pub symbols: ChunkSymbols,
 }
 
-// Code Generator
 
+// Code Generator
 pub struct Compiler {
     builder: ChunkBuilder,
     scope: ScopeTracker,
@@ -548,17 +387,22 @@ impl CodeGenerator<'_> {
     
     ///////// Scopes /////////
     
-    fn emit_begin_scope(&mut self, tag: ScopeTag, symbol: Option<&DebugSymbol>) {
+    fn emit_begin_scope(&mut self, symbol: Option<&DebugSymbol>) {
         let chunk_id = self.chunk_id;
-        self.scope_mut().push_scope(tag, chunk_id, symbol);
+        self.scope_mut().push_scope(symbol);
     }
     
     fn emit_end_scope(&mut self) {
         let scope = self.scope_mut().pop_scope();
         let symbol = scope.debug_symbol();
         
+        // close all upvalues
+        for local in scope.locals().iter().filter(|local| local.captured()) {
+            self.emit_close_upvalue(symbol, local.index());
+        }
+        
         // discard all the locals from the stack
-        let mut discard = scope.locals.len();
+        let mut discard = scope.locals().len();
         while discard > u8::MAX.into() {
             self.emit_instr_byte(symbol, OpCode::DropLocals, u8::MAX);
             discard -= usize::from(u8::MAX);
@@ -571,7 +415,7 @@ impl CodeGenerator<'_> {
     
     // If the local name cannot be found, no instructions are emitted and None is returned
     fn try_emit_load_local(&mut self, symbol: Option<&DebugSymbol>, name: &LocalName) -> Option<u16> {
-        if let Some(index) = self.scope().resolve_local(name).map(|local| local.index) {
+        if let Some(index) = self.scope().resolve_local(name).map(|local| local.index()) {
             if let Ok(index) = u8::try_from(index) {
                 self.emit_instr_byte(symbol, OpCode::LoadLocal, index);
             } else {
@@ -584,6 +428,48 @@ impl CodeGenerator<'_> {
         }
     }
     
+    fn try_emit_load_upval(&mut self, symbol: Option<&DebugSymbol>, name: &LocalName) -> CompileResult<Option<u16>> {
+        if let Some(index) = self.scope_mut().resolve_or_create_upval(name)?.map(|upval| upval.index()) {
+            if let Ok(index) = u8::try_from(index) {
+                self.emit_instr_byte(symbol, OpCode::LoadUpvalue, index);
+            } else {
+                self.emit_instr_data(symbol, OpCode::LoadUpvalue16, &index.to_le_bytes());
+            }
+            
+            Ok(Some(index))
+        } else {
+            Ok(None)
+        }
+    }
+    
+    fn emit_upvalue(&mut self, symbol: Option<&DebugSymbol>, target: UpvalueTarget) {
+        match target {
+            UpvalueTarget::Local(index) => {
+                if let Ok(index) = u8::try_from(index) {
+                    self.emit_instr_byte(symbol, OpCode::InsertUpvalueLocal, index);
+                } else {
+                    self.emit_instr_data(symbol, OpCode::InsertUpvalueLocal16, &index.to_le_bytes());
+                }
+            },
+            
+            UpvalueTarget::Upvalue(index) => {
+                if let Ok(index) = u8::try_from(index) {
+                    self.emit_instr_byte(symbol, OpCode::InsertUpvalueExtern, index);
+                } else {
+                    self.emit_instr_data(symbol, OpCode::InsertUpvalueExtern16, &index.to_le_bytes());
+                }
+            },
+        }
+    }
+    
+    fn emit_close_upvalue(&mut self, symbol: Option<&DebugSymbol>, index: LocalIndex) {
+        if let Ok(index) = u8::try_from(index) {
+            self.emit_instr_byte(symbol, OpCode::CloseUpvalue, index);
+        } else {
+            self.emit_instr_data(symbol, OpCode::CloseUpvalue16, &index.to_le_bytes());
+        }
+    }
+    
     ///////// Statements /////////
     
     fn compile_stmt_with_symbol(&mut self, stmt: &StmtMeta) -> CompileResult<()> {
@@ -648,7 +534,7 @@ impl CodeGenerator<'_> {
         
         let loop_target = self.current_offset();
         
-        self.emit_begin_scope(ScopeTag::Loop, symbol);
+        self.emit_begin_scope(symbol);
         self.compile_stmt_list(body)?;
         self.emit_end_scope();
         
@@ -667,7 +553,7 @@ impl CodeGenerator<'_> {
         
         let loop_target = self.current_offset();
         
-        self.emit_begin_scope(ScopeTag::Loop, symbol);
+        self.emit_begin_scope(symbol);
         self.compile_stmt_list(body)?;
         self.emit_end_scope();
         
@@ -788,15 +674,21 @@ impl CodeGenerator<'_> {
     }
     
     fn compile_name_lookup(&mut self, symbol: Option<&DebugSymbol>, name: &InternSymbol) -> CompileResult<()> {
+        let local_name = LocalName::Symbol(*name);
         
         // Try loading a Local variable
-        if self.try_emit_load_local(symbol, &LocalName::Symbol(*name)).is_none() {
-            
-            // Otherwise, it must be a Global variable
-            self.emit_load_const(symbol, Constant::from(*name))?;
-            self.emit_instr(symbol, OpCode::LoadGlobal);
+        if self.try_emit_load_local(symbol, &local_name).is_some() {
+            return Ok(());
         }
         
+        // Next, try loading an upvalue
+        if self.try_emit_load_upval(symbol, &local_name)?.is_some() {
+            return Ok(());
+        }
+        
+        // Otherwise, it must be a Global variable
+        self.emit_load_const(symbol, Constant::from(*name))?;
+        self.emit_instr(symbol, OpCode::LoadGlobal);
         Ok(())
     }
     
@@ -1025,47 +917,48 @@ impl CodeGenerator<'_> {
         
         // Generate assignment
         
-        if self.scope().local_scope().is_some() {
-            let local = LocalName::Symbol(*name);
+        if !self.scope().is_global_scope() {
+            
+            let local_name = LocalName::Symbol(*name);
             
             // check if the name is found in the local scope...
-            let result = self.scope().resolve_local(&local).map(|local| (local.decl, local.index));
+            let result = self.scope().resolve_local(&local_name);
             
-            if let Some((decl, index)) = result {
-                if decl != DeclType::Mutable {
+            if let Some(local) = result.cloned() {
+                if local.decl() != DeclType::Mutable {
                     return Err(CompileError::from(ErrorKind::CantAssignImmutable));
                 }
-                self.emit_assign_local(symbol, index);
+                
+                self.emit_assign_local(symbol, local.index());
                 
                 return Ok(());
             }
             
-            // TODO upvalues
-            
-            // otherwise search enclosing scopes...
-            // let result = self.scope().resolve_nonlocal(&local).map(|local| (local.decl, local.index));
-            
-            // if let Some((decl, index)) = result {
-            //     if !assign.nonlocal {
-            //         return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
-            //     }
-            //     if decl != DeclType::Mutable {
-            //         return Err(CompileError::from(ErrorKind::CantAssignImmutable));
-            //     }
-            //     self.emit_assign_local(symbol, index);
-                
-            //     return Ok(());
-            // }
+            // nonlocal keyword is not required in the global frame
+            if !assign.nonlocal && !self.scope().is_global_frame() {
+                return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
+            }
             
+            // check if an upvalue is found or can be created...
+            if !self.scope().is_global_frame() {
+                if let Some(upval) = self.scope_mut().resolve_or_create_upval(&local_name)? {
+                    if upval.decl() != DeclType::Mutable {
+                        return Err(CompileError::from(ErrorKind::CantAssignImmutable));
+                    }
+                    
+                    let index = upval.index();
+                    if let Ok(index) = u8::try_from(index) {
+                        self.emit_instr_byte(symbol, OpCode::StoreUpvalue, index);
+                    } else {
+                        self.emit_instr_data(symbol, OpCode::StoreUpvalue16, &index.to_le_bytes());
+                    }
+                    
+                    return Ok(());
+                }
+            }
         }
-        
-        // ...finally, try to assign global
-        
-        // allow assignment to global only if all enclosing scopes permit it
-        if !assign.nonlocal && !self.scope().can_assign_global() {
-            return Err(CompileError::from(ErrorKind::CantAssignNonLocal));
-        }
-        
+
+        // ...finally, try to assign to a global, which are late bound
         self.emit_load_const(symbol, Constant::from(*name))?;
         self.emit_instr(symbol, OpCode::StoreGlobal);
         Ok(())
@@ -1097,7 +990,7 @@ impl CodeGenerator<'_> {
     
     fn compile_block_expression(&mut self, symbol: Option<&DebugSymbol>, label: Option<&Label>, suite: &ExprBlock) -> CompileResult<()> {
         
-        self.emit_begin_scope(ScopeTag::Block, symbol);
+        self.emit_begin_scope(symbol);
         self.compile_expr_block(symbol, suite)?;
         self.emit_end_scope();
         
@@ -1127,7 +1020,7 @@ impl CodeGenerator<'_> {
                 else { Jump::PopIfFalse };
             self.emit_dummy_instr(symbol, branch_jump.dummy_width());
             
-            self.emit_begin_scope(ScopeTag::Branch, symbol);
+            self.emit_begin_scope(symbol);
             self.compile_expr_block(symbol, branch.suite())?;
             self.emit_end_scope();
             
@@ -1146,7 +1039,7 @@ impl CodeGenerator<'_> {
         // else clause
         if let Some(suite) = else_clause {
             
-            self.emit_begin_scope(ScopeTag::Branch, symbol);
+            self.emit_begin_scope(symbol);
             self.compile_expr_block(symbol, suite)?;
             self.emit_end_scope();
             
@@ -1185,7 +1078,7 @@ impl CodeGenerator<'_> {
         
         // and a new local scope
         // don't need to emit new scope instructions, should handled by function call
-        chunk_gen.scope_mut().push_scope(ScopeTag::Function, chunk, symbol);
+        chunk_gen.scope_mut().push_frame(symbol);
         
         // don't need to generate IN_LOCAL instructions for these, the VM should include them automatically
         chunk_gen.scope_mut().insert_local(DeclType::Immutable, LocalName::Receiver)?;
@@ -1205,13 +1098,24 @@ impl CodeGenerator<'_> {
         }
         
         // end the function scope
-        // don't need to emit end scope instructions, will be handled by return
-        chunk_gen.scope_mut().pop_scope();
+        // don't need to drop locals explicitly, that will be done when the VMCallFrame returns
+        let frame = chunk_gen.scope_mut().pop_frame();
+        
+        // however we do still need to close upvalues before we return
+        for local in frame.iter_locals().filter(|local| local.captured()) {
+            chunk_gen.emit_close_upvalue(None, local.index());
+        }
+        
         chunk_gen.finish();
         
         // load the function object as the expression result
         self.emit_load_const(symbol, Constant::Function(chunk_id, function_id))?;
         
+        // compile upvalues
+        for upvalue in frame.upvalues().iter() {
+            self.emit_upvalue(symbol, upvalue.target());
+        }
+        
         Ok(())
     }
     
