@@ -1,12 +1,11 @@
 use std::fmt;
 use std::fmt::{Write, Formatter};
 use std::iter;
-use std::collections::HashMap;
 use string_interner::Symbol as _;
 
 use crate::language::FloatType;
 use crate::codegen::OpCode;
-use crate::codegen::chunk::{UnloadedProgram, Chunk, ChunkID};
+use crate::codegen::chunk::{UnloadedProgram, Chunk};
 use crate::codegen::consts::{Constant, ConstID};
 use crate::debug::symbol::{DebugSymbol, DebugSymbolTable, ResolvedSymbol, ResolvedSymbolTable, ChunkSymbols};
 use crate::debug::symbol::errors::SymbolResolutionError;
@@ -113,7 +112,7 @@ impl<'c, 's> Disassembler<'c, 's> {
                 
                 OpCode::Drop | OpCode::DropLocals => {
                     let len = instr[1];
-                    write!(line, "{:16} {: >4}    ", opcode, len)?;
+                    write!(line, "{:16} {: >4}", opcode, len)?;
                 }
                 
                 OpCode::LoadConst => {
@@ -129,14 +128,41 @@ impl<'c, 's> Disassembler<'c, 's> {
                 },
                 
                 OpCode::StoreLocal | OpCode::LoadLocal => {
-                    let offset = instr[1];
-                    write!(line, "{:16} {: >4}    ", opcode, offset)?;
+                    let index = instr[1];
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
                 },
                 OpCode::StoreLocal16 | OpCode::LoadLocal16 => {
-                    let offset =  u16::from_le_bytes(instr[1..=2].try_into().unwrap());
-                    write!(line, "{:16} {: >4}    ", opcode, offset)?;
+                    let index =  u16::from_le_bytes(instr[1..=2].try_into().unwrap());
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
                 },
                 
+                OpCode::InsertUpvalueLocal | OpCode::InsertUpvalueExtern => {
+                    let index = instr[1];
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                OpCode::InsertUpvalueLocal16 | OpCode::InsertUpvalueExtern16 => {
+                    let index =  u16::from_le_bytes(instr[1..=2].try_into().unwrap());
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                
+                OpCode::StoreUpvalue | OpCode::LoadUpvalue => {
+                    let index = instr[1];
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                OpCode::StoreUpvalue16 | OpCode::LoadUpvalue16 => {
+                    let index =  u16::from_le_bytes(instr[1..=2].try_into().unwrap());
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                
+                OpCode::CloseUpvalue => {
+                    let index = instr[1];
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                OpCode::CloseUpvalue16 => {
+                    let index =  u16::from_le_bytes(instr[1..=2].try_into().unwrap());
+                    write!(line, "{:16} {: >4}    ", opcode, index)?;
+                }
+                
                 OpCode::Tuple => {
                     let len = instr[1];
                     write!(line, "{:16} {: >4}    ", opcode, len)?;
