@@ -3,8 +3,7 @@ use std::error::Error;
 
 use crate::utils;
 use crate::runtime::Variant;
-use crate::runtime::strings::StringSymbol;
-use crate::runtime::types::function::Signature;
+use crate::runtime::function::Signature;
 use crate::debug::traceback::{TraceSite, Traceback};
 
 // TODO box error
@@ -91,7 +90,7 @@ impl fmt::Display for RuntimeError {
             ErrorKind::Other => String::new(),
         };
         
-        utils::format_error(fmt, "Runtime error", Some(message.as_str()), self.source())
+        utils::format_error(fmt, "Runtime error", Some(&message), self.source())
     }
 }
 
