@@ -14,7 +14,7 @@ use crate::source::ModuleSource;
 use crate::language::FloatType;
 use crate::runtime::{Variant, HashMap, DefaultBuildHasher};
 use crate::runtime::gc::{GC, GCTrace};
-use crate::runtime::types::function::Function;
+use crate::runtime::function::Function;
 use crate::runtime::strings::StringSymbol;
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
@@ -121,6 +121,7 @@ impl GCTrace for GlobalEnv { }
 #[derive(Debug)]
 pub struct Module {
     ident: ModuleIdent,
+    display: String,
     source: Option<ModuleSource>,
     data: ProgramData,
     globals: GC<GlobalEnv>,
@@ -139,8 +140,11 @@ impl Module {
             if let Some(source) = source.as_ref() { ModuleIdent::from(source) }
             else { ModuleIdent::from(globals) };
         
+        let display = ident.to_string();
+        
         let module = Self {
             ident,
+            display,
             source,
             data,
             globals,
@@ -177,6 +181,12 @@ impl Module {
     }
 }
 
+impl fmt::Display for Module {
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt.write_str(&self.display)
+    }
+}
+
 
 /// Used to uniquely identify a module
 #[derive(Debug, PartialEq, Eq, Hash)]
@@ -220,7 +230,8 @@ impl fmt::Display for ModuleIdent {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::SourcePath(path) => {
-                let prefix = std::env::current_dir().ok();
+                let prefix = std::env::current_dir().ok()
+                    .map(|pwd| pwd.canonicalize().ok()).flatten();
                 
                 let path = prefix.map(|prefix| path.strip_prefix(prefix).ok()).flatten()
                     .unwrap_or(path);
