@@ -1,12 +1,8 @@
 use std::fmt;
-use crate::codegen::ChunkID;
-use crate::runtime::Variant;
-use crate::runtime::errors::ExecResult;
 
 pub mod operator;
 pub mod metatable;
 pub mod primitive;
-pub mod function;
 
 
 // Type tag for Sphinx's "primitive" types
