@@ -1,13 +1,13 @@
 use std::fmt;
-use std::rc::Rc;
+use std::cell::Cell;
 use std::hash::{Hash, Hasher};
 use std::cmp::{PartialEq, Eq};
 use crate::language::{IntType, FloatType};
 use crate::runtime::types::Type;
 use crate::runtime::types::metatable::Metatable;
-use crate::runtime::types::function::{Function, NativeFunction, Call, Invoke};
+use crate::runtime::function::{Function, NativeFunction, Call, Invoke};
 use crate::runtime::types::primitive::*;
-use crate::runtime::strings::{StringSymbol, STRING_TABLE};
+use crate::runtime::strings::StringSymbol;
 use crate::runtime::gc::{GC, GCArray, GCTrace};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
@@ -148,6 +148,9 @@ impl From<NativeFunction> for Variant {
 
 impl GCTrace for Variant { }
 
+impl GCTrace for Cell<Variant> { }
+
+
 // extract the GC handle for GC'd types
 impl TryFrom<Variant> for GC<dyn GCTrace> {
     type Error = Variant;
